@@ -144,7 +144,7 @@ class RAIInsights(RAIBaseInsights):
             self.predict_output = model.predict(
                 test.drop(columns=[target_column]))
             if hasattr(model, SKLearn.PREDICT_PROBA):
-                self.predict_proba_output = model.predict(
+                self.predict_proba_output = model.predict_proba(
                     test.drop(columns=[target_column]))
             else:
                 self.predict_proba_output = None
@@ -287,6 +287,12 @@ class RAIInsights(RAIBaseInsights):
                     'The serializer should be serializable via pickle')
 
         if isinstance(train, pd.DataFrame) and isinstance(test, pd.DataFrame):
+            if len(train) <= 0 or len(test) <= 0:
+                raise UserConfigValidationException(
+                    'Either of the train/test are empty. '
+                    'Please provide non-empty dataframes for train '
+                    'and test sets.'
+                )
             if test.shape[0] > maximum_rows_for_test:
                 msg_fmt = 'The test data has {0} rows, ' +\
                     'but limit is set to {1} rows. ' +\
@@ -298,7 +304,7 @@ class RAIInsights(RAIBaseInsights):
                 )
 
             if len(set(train.columns) - set(test.columns)) != 0 or \
-                    len(set(test.columns) - set(train.columns)):
+                    len(set(test.columns) - set(train.columns)) != 0:
                 raise UserConfigValidationException(
                     'The features in train and test data do not match')
 
@@ -318,7 +324,8 @@ class RAIInsights(RAIBaseInsights):
                             target_column)
                     )
 
-                difference_set = set(categorical_features) - set(train.columns)
+                difference_set = set(categorical_features) - set(
+                    train.drop(columns=[target_column]).columns)
                 if len(difference_set) > 0:
                     message = ("Feature names in categorical_features "
                                "do not exist in train data: "
@@ -342,6 +349,18 @@ class RAIInsights(RAIBaseInsights):
                             "Please check your test data.".format(column)
                         )
 
+            train_features = train.drop(columns=[target_column]).columns
+            numeric_features = train.drop(
+                columns=[target_column]).select_dtypes(
+                    include='number').columns.tolist()
+            string_features_set = set(train_features) - set(numeric_features)
+            if len(string_features_set - set(categorical_features)) > 0:
+                raise UserConfigValidationException(
+                    "The following string features were not "
+                    "identified as categorical features: {0}".format(
+                        string_features_set - set(categorical_features))
+                )
+
             if classes is not None and task_type == \
                     ModelTask.CLASSIFICATION:
                 if len(set(train[target_column].unique()) -
